@@ -18,7 +18,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#  include <config.h>
 #endif /* HAVE_CONFIG_H */
 
 #include <stdlib.h>
@@ -105,10 +105,7 @@ typedef enum {
 
     IN_COMPLETE_TEST_SUITE,
 
-    IN_COMPLETE_SUCCESS,
-
-    IN_CRASHED,
-    IN_CRASHED_BACKTRACE
+    IN_COMPLETE_SUCCESS
 } ParseState;
 
 #define CUT_STREAM_PARSER_GET_PRIVATE(obj)                              \
@@ -228,7 +225,6 @@ struct _CutStreamParserPrivate
     gchar *option_value;
     gboolean complete_success;
     gboolean stream_success;
-    gchar *crashed_backtrace;
 };
 
 #define PUSH_STATE(priv, state)                                 \
@@ -411,7 +407,6 @@ cut_stream_parser_init (CutStreamParser *stream_parser)
     priv->test_result = NULL;
     priv->test_iterator_result = NULL;
     priv->test_case_result = NULL;
-    priv->crashed_backtrace = NULL;
 
     priv->result = NULL;
     priv->backtrace = NULL;
@@ -732,11 +727,6 @@ dispose (GObject *object)
         priv->option_value = NULL;
     }
 
-    if (priv->crashed_backtrace) {
-        g_free(priv->crashed_backtrace);
-        priv->crashed_backtrace = NULL;
-    }
-
     G_OBJECT_CLASS(cut_stream_parser_parent_class)->dispose(object);
 }
 
@@ -946,8 +936,6 @@ start_stream (CutStreamParserPrivate *priv,
         PUSH_STATE(priv, IN_COMPLETE_TEST_SUITE);
     } else if (g_str_equal("success", element_name)) {
         PUSH_STATE(priv, IN_STREAM_SUCCESS);
-    } else if (g_str_equal("crashed", element_name)) {
-        PUSH_STATE(priv, IN_CRASHED);
     } else {
         invalid_element(context, error);
     }
@@ -1317,17 +1305,6 @@ start_complete_test_suite (CutStreamParserPrivate *priv,
     }
 }
 
-static void
-start_crashed (CutStreamParserPrivate *priv, GMarkupParseContext *context,
-               const gchar *element_name, GError **error)
-{
-    if (g_str_equal("backtrace", element_name)) {
-        PUSH_STATE(priv, IN_CRASHED_BACKTRACE);
-    } else {
-        invalid_element(context, error);
-    }
-}
-
 static void
 start_test_object (CutStreamParserPrivate *priv, GMarkupParseContext *context,
                    const gchar *element_name, GError **error)
@@ -1540,9 +1517,6 @@ start_element_handler (GMarkupParseContext *context,
       case IN_COMPLETE_TEST_SUITE:
         start_complete_test_suite(priv, context, element_name, error);
         break;
-      case IN_CRASHED:
-        start_crashed(priv, context, element_name, error);
-        break;
       case IN_TEST_SUITE:
       case IN_TEST_CASE:
       case IN_TEST_ITERATOR:
@@ -1656,9 +1630,9 @@ end_result_backtrace (CutStreamParser *parser, CutStreamParserPrivate *priv,
     if (!priv->backtrace)
         return;
 
+    priv->backtrace = g_list_reverse(priv->backtrace);
     if (priv->result)
-        cut_test_result_set_backtrace(priv->result,
-                                      g_list_reverse(priv->backtrace));
+        cut_test_result_set_backtrace(priv->result, priv->backtrace);
 
     g_list_foreach(priv->backtrace, (GFunc)g_object_unref, NULL);
     g_list_free(priv->backtrace);
@@ -2058,22 +2032,6 @@ end_complete_test_suite (CutStreamParser *parser, CutStreamParserPrivate *priv,
     priv->complete_success = TRUE;
 }
 
-static void
-end_crashed (CutStreamParser *parser, CutStreamParserPrivate *priv,
-             GMarkupParseContext *context,
-             const gchar *element_name, GError **error)
-{
-    if (priv->run_context)
-        g_signal_emit_by_name(priv->run_context,
-                              "crashed", priv->crashed_backtrace);
-
-    if (!priv->crashed_backtrace)
-        return;
-
-    g_free(priv->crashed_backtrace);
-    priv->crashed_backtrace = NULL;
-}
-
 static void
 end_element_handler (GMarkupParseContext *context,
                      const gchar         *element_name,
@@ -2162,9 +2120,6 @@ end_element_handler (GMarkupParseContext *context,
       case IN_COMPLETE_TEST_SUITE:
         end_complete_test_suite(parser, priv, context, element_name, error);
         break;
-      case IN_CRASHED:
-        end_crashed(parser, priv, context, element_name, error);
-        break;
       default:
         break;
     }
@@ -2185,6 +2140,8 @@ result_name_to_status (const gchar *name)
         return CUT_TEST_RESULT_NOTIFICATION;
     else if (g_str_equal(name, "omission"))
         return CUT_TEST_RESULT_OMISSION;
+    else if (g_str_equal(name, "crash"))
+        return CUT_TEST_RESULT_CRASH;
 
     return CUT_TEST_RESULT_INVALID;
 }
@@ -2395,29 +2352,27 @@ text_result_backtrace_entry_info (CutStreamParserPrivate *priv,
                                   const gchar *text, gsize text_len,
                                   GError **error)
 {
-    if (g_str_has_suffix(text, "()")) {
+    const gchar *info_start;
+
+    info_start = strstr(text, "():");
+    if (info_start) {
+        gchar *function;
+
+        function = g_strndup(text, info_start - text);
+        info_start += strlen("():");
+        while (info_start[0] && info_start[0] == ' ')
+            info_start++;
+        cut_backtrace_entry_set_function(priv->backtrace_entry, function);
+        cut_backtrace_entry_set_info(priv->backtrace_entry, info_start);
+        g_free(function);
+    } else if (g_str_has_suffix(text, "()")) {
         gchar *function;
 
         function = g_strndup(text, strlen(text) - 2);
         cut_backtrace_entry_set_function(priv->backtrace_entry, function);
         g_free(function);
     } else {
-        const gchar *info_start;
-
-        info_start = strstr(text, "():");
-        if (info_start) {
-            gchar *function;
-
-            function = g_strndup(text, info_start - text);
-            info_start += strlen("():");
-            while (info_start[0] && info_start[0] == ' ')
-                info_start++;
-            cut_backtrace_entry_set_function(priv->backtrace_entry, function);
-            cut_backtrace_entry_set_info(priv->backtrace_entry, info_start);
-            g_free(function);
-        } else {
-            cut_backtrace_entry_set_info(priv->backtrace_entry, text);
-        }
+        cut_backtrace_entry_set_info(priv->backtrace_entry, text);
     }
 }
 
@@ -2551,14 +2506,6 @@ text_stream_success (CutStreamParserPrivate *priv, GMarkupParseContext *context,
     }
 }
 
-static void
-text_crashed_backtrace (CutStreamParserPrivate *priv,
-                        GMarkupParseContext *context,
-                        const gchar *text, gsize text_len, GError **error)
-{
-    priv->crashed_backtrace = g_strdup(text);
-}
-
 static void
 text_handler (GMarkupParseContext *context,
               const gchar         *text,
@@ -2638,9 +2585,6 @@ text_handler (GMarkupParseContext *context,
       case IN_STREAM_SUCCESS:
         text_stream_success(priv, context, text, text_len, error);
         break;
-      case IN_CRASHED_BACKTRACE:
-        text_crashed_backtrace(priv, context, text, text_len, error);
-        break;
       default:
         break;
     }
