@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2007-2009  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2007-2010  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -30,6 +30,7 @@
 #include <glib/gstdio.h>
 #ifdef G_OS_WIN32
 #  include <windows.h>
+#  include <winsock2.h>
 #  include <io.h>
 #  define close _close
 #endif
@@ -46,6 +47,21 @@
 #include "../gcutter/gcut-error.h"
 #include "../gcutter/gcut-assertions-helper.h"
 
+#ifndef CUT_DISABLE_SOCKET_SUPPORT
+#  ifdef HAVE_WINSOCK2_H
+#    include <winsock2.h>
+#    include <ws2tcpip.h>
+#  else
+#    include <sys/types.h>
+#    include <sys/socket.h>
+#    include <netinet/in.h>
+#    include <arpa/inet.h>
+#    ifdef HAVE_SYS_UN_H
+#      include <sys/un.h>
+#    endif
+#  endif
+#endif
+
 gchar *
 cut_utils_create_regex_pattern (const gchar *string)
 {
@@ -237,6 +253,153 @@ cut_utils_inspect_string (const gchar *string)
         return "(null)";
 }
 
+#ifndef CUT_DISABLE_SOCKET_SUPPORT
+gboolean
+cut_utils_equal_sockaddr (const struct sockaddr *address1,
+                          const struct sockaddr *address2)
+{
+    if (address1 == address2)
+        return TRUE;
+    if (!address1 || !address2)
+        return FALSE;
+    if (address1->sa_family != address2->sa_family)
+        return FALSE;
+
+    switch (address1->sa_family) {
+#ifdef HAVE_SYS_UN_H
+    case AF_UNIX:
+    {
+        struct sockaddr_un *address_unix1 = (struct sockaddr_un *)address1;
+        struct sockaddr_un *address_unix2 = (struct sockaddr_un *)address2;
+
+        return cut_utils_equal_string(address_unix1->sun_path,
+                                      address_unix2->sun_path);
+        break;
+    }
+#endif
+    case AF_INET:
+    {
+        struct sockaddr_in *address_inet1 = (struct sockaddr_in *)address1;
+        struct sockaddr_in *address_inet2 = (struct sockaddr_in *)address2;
+
+        if (address_inet1->sin_addr.s_addr != address_inet2->sin_addr.s_addr)
+            return FALSE;
+        if (address_inet1->sin_port != address_inet2->sin_port)
+            return FALSE;
+
+        return TRUE;
+        break;
+    }
+    case AF_INET6:
+    {
+        struct sockaddr_in6 *address_inet6_1 = (struct sockaddr_in6 *)address1;
+        struct sockaddr_in6 *address_inet6_2 = (struct sockaddr_in6 *)address2;
+
+        if (memcmp(address_inet6_1->sin6_addr.s6_addr,
+                   address_inet6_2->sin6_addr.s6_addr,
+                   sizeof(address_inet6_1->sin6_addr.s6_addr)) != 0)
+            return FALSE;
+        if (address_inet6_1->sin6_port != address_inet6_2->sin6_port)
+            return FALSE;
+
+        return TRUE;
+        break;
+    }
+    case AF_UNSPEC:
+        return TRUE;
+        break;
+    default:
+        return FALSE;
+        break;
+    }
+
+    return FALSE;
+}
+
+#ifdef G_OS_WIN32
+static const char *
+inet_ntop (int address_family, const void *source,
+           char *destination, socklen_t destination_length)
+{
+    DWORD socket_address_length;
+    DWORD winsock_destination_length = destination_length;
+
+    switch (address_family) {
+    case AF_INET:
+        socket_address_length = sizeof(struct sockaddr_in);
+        break;
+    case AF_INET6:
+        socket_address_length = sizeof(struct sockaddr_in6);
+        break;
+    default:
+        return NULL;
+        break;
+    }
+
+    if (WSAAddressToString((LPSOCKADDR)source, socket_address_length, NULL,
+                           destination, &winsock_destination_length)) {
+        return NULL;
+    } else {
+        return destination;
+    }
+}
+#endif
+
+gchar *
+cut_utils_inspect_sockaddr (const struct sockaddr *address)
+{
+    gchar *spec = NULL;
+
+    if (!address)
+        return g_strdup("(null)");
+
+    switch (address->sa_family) {
+#ifdef HAVE_SYS_UN_H
+    case AF_UNIX:
+    {
+        struct sockaddr_un *address_unix = (struct sockaddr_un *)address;
+        spec = g_strdup_printf("unix:%s", address_unix->sun_path);
+        break;
+    }
+#endif
+    case AF_INET:
+    {
+        struct sockaddr_in *address_inet = (struct sockaddr_in *)address;
+        gchar ip_address_string[INET_ADDRSTRLEN];
+
+        if (inet_ntop(AF_INET, &address_inet->sin_addr,
+                      (gchar *)ip_address_string, INET_ADDRSTRLEN)) {
+            spec = g_strdup_printf("inet:%s:%d",
+                                   ip_address_string,
+                                   g_ntohs(address_inet->sin_port));
+        }
+        break;
+    }
+    case AF_INET6:
+    {
+        struct sockaddr_in6 *address_inet6 = (struct sockaddr_in6 *)address;
+        gchar ip_address_string[INET6_ADDRSTRLEN];
+
+        if (inet_ntop(AF_INET6, &address_inet6->sin6_addr,
+                      (gchar *)ip_address_string, INET6_ADDRSTRLEN)) {
+            spec = g_strdup_printf("inet6:[%s]:%d",
+                                   ip_address_string,
+                                   g_ntohs(address_inet6->sin6_port));
+        }
+        break;
+    }
+    case AF_UNSPEC:
+        spec = g_strdup("unknown");
+        break;
+    default:
+        spec = g_strdup_printf("unexpected:%d", address->sa_family);
+        break;
+    }
+
+    return spec;
+}
+#endif
+
 gboolean
 cut_utils_path_exist (const gchar *path)
 {
@@ -545,6 +708,12 @@ cut_utils_remove_path_recursive_force (const gchar *path)
     cut_utils_remove_path_recursive(path, NULL);
 }
 
+void
+cut_utils_make_directory_recursive_force (const gchar *path)
+{
+    g_mkdir_with_parents(path, 0700);
+}
+
 gchar *
 cut_utils_append_diff (const gchar *message, const gchar *from, const gchar *to)
 {
