@@ -10,7 +10,12 @@ require 'rd-visitor-util'
 require "rt/rtparser"
 
 require "gettext"
-require "gettext/poparser"
+begin
+  require "gettext/tools"
+rescue LoadError
+  require "gettext/poparser"
+  GetText::MOFile = MOFile
+end
 
 module RD
   class RD2RefEntryVisitor < RDVisitor
@@ -67,8 +72,9 @@ module RD
     def apply_to_Headline(element, title)
       id = nil
       if /\A\s*\[(.+?)\]\s*/ =~ title.first
-        id = $1
+        local_id = $1
         title[0] = $POSTMATCH
+        id = "#{ref_entry_id}.#{local_id}"
       end
       case element.level
       when 1
@@ -248,6 +254,10 @@ module RD
       tag("varname", {}, *contents)
     end
 
+    def apply_to_Keyboard(element, contents)
+      tag("command", {}, *contents)
+    end
+
     def apply_to_StringElement(element)
       apply_to_String(remove_newline(element.content))
     end
@@ -266,7 +276,7 @@ module RD
     end
 
     def make_translate_data
-      data = MOFile.new
+      data = GetText::MOFile.new
       if @@po
         parser = GetText::PoParser.new
         parser.parse(File.read(@@po), data)
@@ -290,7 +300,7 @@ module RD
     end
 
     def term_index_id(term)
-      first_sentence = term.to_s.split(/\s/, 2)[0]
+      first_sentence = term.to_s.split(/[\s\(=]/, 2)[0]
       first_sentence.gsub(/[._]/, "-")
     end
 
